@@ -3,7 +3,7 @@ package App::ModuleBuildTiny;
 use 5.010;
 use strict;
 use warnings;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 
 use Exporter 5.57 'import';
 our @EXPORT = qw/modulebuildtiny/;
@@ -19,7 +19,7 @@ use File::Path qw/mkpath/;
 use File::Slurper qw/write_text write_binary read_binary/;
 use File::Spec::Functions qw/catfile catdir rel2abs/;
 use Getopt::Long 2.36 'GetOptionsFromArray';
-use JSON::PP qw/encode_json decode_json/;
+use JSON::PP qw/decode_json/;
 use Module::Runtime 'require_module';
 use Pod::Simple::Text 3.23;
 use Text::Template;
@@ -28,7 +28,7 @@ use App::ModuleBuildTiny::Dist;
 
 use Env qw/$AUTHOR_TESTING $RELEASE_TESTING $AUTOMATED_TESTING $EXTENDED_TESTING $NONINTERACTIVE_TESTING $SHELL $HOME $USERPROFILE/;
 
-Getopt::Long::Configure(qw/require_order pass_through gnu_compat/);
+Getopt::Long::Configure(qw/require_order gnu_compat/);
 
 sub prompt {
 	my($mess, $def) = @_;
@@ -101,7 +101,17 @@ sub write_json {
 	my ($filename, $content) = @_;
 	my $dirname = dirname($filename);
 	mkdir $dirname if not -d $dirname;
-	return write_binary($filename, encode_json($content));
+	my $json = JSON::PP->new->utf8->pretty->canonical->encode($content);
+	return write_binary($filename, $json);
+}
+
+sub bump_versions {
+	my (%opts) = @_;
+	require App::RewriteVersion;
+	my $app = App::RewriteVersion->new(%opts);
+	my $trial = delete $opts{trial};
+	my $new_version = defined $opts{version} ? delete $opts{version} : $app->bump_version($app->current_version);
+	$app->rewrite_versions($new_version, is_trial => $trial);
 }
 
 my @config_items = (
@@ -113,7 +123,7 @@ my @config_items = (
 my %actions = (
 	dist => sub {
 		my @arguments = @_;
-		GetOptionsFromArray(\@arguments, \my %opts, qw/trial verbose!/);
+		GetOptionsFromArray(\@arguments, \my %opts, qw/trial verbose!/) or exit 2;
 		my $dist = App::ModuleBuildTiny::Dist->new(%opts);
 		die "Trial mismatch" if $opts{trial} && $dist->release_status ne 'testing';
 		my $name = $dist->meta->name . '-' . $dist->meta->version;
@@ -123,7 +133,7 @@ my %actions = (
 	},
 	distdir => sub {
 		my @arguments = @_;
-		GetOptionsFromArray(\@arguments, \my %opts, qw/trial verbose!/);
+		GetOptionsFromArray(\@arguments, \my %opts, qw/trial verbose!/) or exit 2;
 		my $dist = App::ModuleBuildTiny::Dist->new(%opts);
 		die "Trial mismatch" if $opts{trial} && $dist->release_status ne 'testing';
 		$dist->write_dir($dist->meta->name . '-' . $dist->meta->version, $opts{verbose});
@@ -133,13 +143,13 @@ my %actions = (
 		my @arguments = @_;
 		$AUTHOR_TESTING = 1;
 		GetOptionsFromArray(\@arguments, 'release!' => \$RELEASE_TESTING, 'author!' => \$AUTHOR_TESTING, 'automated!' => \$AUTOMATED_TESTING,
-			'extended!' => \$EXTENDED_TESTING, 'non-interactive!' => \$NONINTERACTIVE_TESTING);
+			'extended!' => \$EXTENDED_TESTING, 'non-interactive!' => \$NONINTERACTIVE_TESTING) or exit 2;
 		my $dist = App::ModuleBuildTiny::Dist->new;
 		return $dist->run(command => [ $Config{perlpath}, 'Build', 'test' ], build => 1);
 	},
 	upload => sub {
 		my @arguments = @_;
-		GetOptionsFromArray(\@arguments, \my %opts, qw/trial config=s silent/);
+		GetOptionsFromArray(\@arguments, \my %opts, qw/trial config=s silent/) or exit 2;
 
 		my $dist = App::ModuleBuildTiny::Dist->new;
 		$dist->run(command => [ $Config{perlpath}, 'Build', 'test' ], build => 1) or return 1;
@@ -156,19 +166,19 @@ my %actions = (
 	run => sub {
 		my @arguments = @_;
 		croak "No arguments given to run" if not @arguments;
-		GetOptionsFromArray(\@arguments, 'build!' => \(my $build = 1));
+		GetOptionsFromArray(\@arguments, 'build!' => \(my $build = 1)) or exit 2;
 		my $dist = App::ModuleBuildTiny::Dist->new();
 		return $dist->run(command => \@arguments, build => $build);
 	},
 	shell => sub {
 		my @arguments = @_;
-		GetOptionsFromArray(\@arguments, 'build!' => \my $build);
+		GetOptionsFromArray(\@arguments, 'build!' => \my $build) or exit 2;
 		my $dist = App::ModuleBuildTiny::Dist->new();
 		return $dist->run(command => [ $SHELL ], build => $build);
 	},
 	listdeps => sub {
 		my @arguments = @_;
-		GetOptionsFromArray(\@arguments, \my %opts, qw/json only_missing|only-missing|missing omit_core|omit-core=s author versions/);
+		GetOptionsFromArray(\@arguments, \my %opts, qw/json only_missing|only-missing|missing omit_core|omit-core=s author versions/) or exit 2;
 		my $dist = App::ModuleBuildTiny::Dist->new;
 
 		require CPAN::Meta::Prereqs::Filter;
@@ -197,15 +207,29 @@ my %actions = (
 	},
 	regenerate => sub {
 		my @arguments = @_;
-		GetOptionsFromArray(\@arguments, \my %opts, qw/trial/);
+		GetOptionsFromArray(\@arguments, \my %opts, qw/trial bump version=s verbose dry_run/) or exit 2;
 		my %files = map { $_ => 1 } @arguments ? @arguments : qw/Build.PL META.json META.yml MANIFEST LICENSE README/;
 
-		my $dist = App::ModuleBuildTiny::Dist->new(%opts, regenerate => \%files);
-		for my $filename ($dist->files) {
-			write_binary($filename, $dist->get_file($filename)) if $dist->is_generated($filename);
+		if ($opts{bump}) {
+			bump_versions(%opts);
+		}
+
+		if (!$opts{dry_run}) {
+			my $dist = App::ModuleBuildTiny::Dist->new(%opts, regenerate => \%files);
+			for my $filename ($dist->files) {
+				write_binary($filename, $dist->get_file($filename)) if $dist->is_generated($filename);
+			}
 		}
 		return 0;
 	},
+	scan => sub {
+		my @arguments = @_;
+		my %opts = (sanitize => 1);
+		GetOptionsFromArray(\@arguments, \%opts, qw/omit_core|omit-core=s sanitize omit=s@/) or exit 2;
+		my $dist = App::ModuleBuildTiny::Dist->new(regenerate => { 'META.json' => 1 });
+		my $prereqs = $dist->scan_prereqs(%opts);
+		write_json('prereqs.json', $prereqs->as_string_hash);
+	},
 	configure => sub {
 		my @arguments = @_;
 		my $config_file = get_config();
@@ -229,6 +253,13 @@ my %actions = (
 			}
 			write_json($config_file, $config);
 		}
+		elsif ($mode eq 'list') {
+			my $config = -f $config_file ? read_json($config_file) : {};
+			for my $item (@config_items) {
+				my ($key, $description, $default) = @{$item};
+				printf "%s: %s\n", ucfirst $key, $config->{$key} // '(undefined)';
+			}
+		}
 		elsif ($mode eq 'reset') {
 			return not unlink $config_file;
 		}
@@ -250,7 +281,7 @@ my %actions = (
 			version => '0.001',
 			dirname => $distname,
 		);
-		GetOptionsFromArray(\@arguments, \%args, qw/author=s email=s version=s abstract=s license=s dirname=s/);
+		GetOptionsFromArray(\@arguments, \%args, qw/author=s email=s version=s abstract=s license=s dirname=s/) or exit 2;
 
 		my $license = create_license_for(delete $args{license}, $args{author});
 
@@ -281,10 +312,6 @@ sub modulebuildtiny {
 
 App::ModuleBuildTiny - A standalone authoring tool for Module::Build::Tiny
 
-=head1 VERSION
-
-version 0.029
-
 =head1 DESCRIPTION
 
 App::ModuleBuildTiny contains the implementation of the L<mbtiny> tool.
@@ -303,28 +330,6 @@ The actions are documented in the L<mbtiny> documentation.
 
 =head1 SEE ALSO
 
-=head2 Helpers
-
-=over 4
-
-=item * L<scan-prereqs-cpanfile|scan-prereqs-cpanfile>
-
-A tool to automatically generate a L<cpanfile> for you.
-
-=item * L<cpan-upload|cpan-upload>
-
-A program that facilitates upload the tarball as produced by C<mbtiny>.
-
-=item * L<perl-reversion|https://metacpan.org/pod/distribution/Perl-Version/examples/perl-reversion>
-
-A tool to bump the version in your modules.
-
-=item * L<perl-bump-version|perl-bump-version>
-
-An alternative tool to bump the version in your modules
-
-=back
-
 =head2 Similar programs
 
 =over 4
