@@ -3,7 +3,7 @@ package App::ModuleBuildTiny::Dist;
 use 5.010;
 use strict;
 use warnings;
-our $VERSION = '0.029';
+our $VERSION = '0.030';
 
 use CPAN::Meta;
 use Carp qw/croak/;
@@ -32,18 +32,12 @@ sub find {
 }
 
 sub mbt_version {
-	if (find(qr/\.PL$/, 'lib')) {
-		return '0.039';
-	}
-	elsif (find(qr/\.xs$/, 'lib')) {
-		return '0.036';
-	}
-	return '0.034';
+	return '0.039';
 }
 
 sub prereqs_for {
 	my ($meta, $phase, $type, $module, $default) = @_;
-	return $meta->effective_prereqs->requirements_for($phase, $type)->requirements_for_module($module) || $default || 0;
+	return $meta->effective_prereqs->requirements_for($phase, $type)->requirements_for_module($module) // $default // 0;
 }
 
 sub uptodate {
@@ -140,19 +134,62 @@ sub checkmeta {
 	die sprintf "Version mismatch between module and meta, did you forgot to run regenerate? (%s versus %s)", $detected_version, $meta_version if $detected_version != $meta_version;
 }
 
+sub scan_files {
+	my ($files, $omit) = @_;
+	my $combined = CPAN::Meta::Requirements->new;
+	require Perl::PrereqScanner;
+	my $scanner = Perl::PrereqScanner->new;
+	for my $file (@{$files}) {
+		my $prereqs = $scanner->scan_file($file);
+		$combined->add_requirements($prereqs);
+	}
+	$combined->clear_requirement($_) for @{$omit};
+	return $combined
+}
+
+sub scan_prereqs {
+	my ($self, %opts) = @_;
+	my (@runtime_files, @test_files);
+	File::Find::find(sub { push @runtime_files, $File::Find::name if -f && /\.pm$/ }, 'lib');
+	File::Find::find(sub { push @runtime_files, $File::Find::name if -f }, 'script');
+	File::Find::find(sub { push @test_files, $File::Find::name if -f && /\.(t|pm)$/ }, 't');
+
+	my @omit = (@{ $opts{omit} || [] }, keys %{ $self->{meta}->provides });
+	my $runtime = scan_files(\@runtime_files, \@omit);
+	my $test = scan_files(\@test_files, \@omit);
+
+	my $prereqs = CPAN::Meta::Prereqs->new({
+		runtime   => { requires => $runtime->as_string_hash },
+		test      => { requires => $test->as_string_hash },
+		configure => { requires => { 'Module::Build::Tiny' => mbt_version() } },
+		develop   => { requires => { 'App::ModuleBuildTiny' => $VERSION } },
+	});
+	require CPAN::Meta::Prereqs::Filter;
+	return CPAN::Meta::Prereqs::Filter::filter_prereqs($prereqs, %opts);
+}
+
 sub load_prereqs {
+	my @prereqs;
 	if (-f 'prereqs.json') {
-		return load_jsonyaml('prereqs.json');
+		push @prereqs, load_jsonyaml('prereqs.json');
 	}
-	elsif (-f 'prereqs.yml') {
-		return load_jsonyaml('prereqs.yml');
+	if (-f 'prereqs.yml') {
+		push @prereqs, load_jsonyaml('prereqs.yml');
 	}
-	elsif (-f 'cpanfile') {
+	if (-f 'cpanfile') {
 		require Module::CPANfile;
-		return Module::CPANfile->load('cpanfile')->prereq_specs;
+		push @prereqs, Module::CPANfile->load('cpanfile')->prereq_specs;
+	}
+	if (@prereqs == 1) {
+		return $prereqs[0];
+	}
+	elsif (@prereqs == 0) {
+		return {};
 	}
 	else {
-		return {};
+		@prereqs = map { CPAN::Meta::Prereqs->new($_) } @prereqs;
+		my $prereqs = $prereqs[0]->with_merged_prereqs([ @prereqs[1..$#prereqs] ]);
+		$prereqs->as_string_hash;
 	}
 }
 
@@ -174,7 +211,6 @@ sub new {
 
 		my $prereqs = load_prereqs();
 		$prereqs->{configure}{requires}{'Module::Build::Tiny'} //= mbt_version();
-		$prereqs->{develop}{requires}{'App::ModuleBuildTiny'} //= $VERSION;
 
 		my $metahash = {
 			name           => $distname,
@@ -233,7 +269,7 @@ sub new {
 	my $dist_name = $meta->name;
 	$files{'Build.PL'} //= do {
 		my $minimum_mbt  = prereqs_for($meta, qw/configure requires Module::Build::Tiny/);
-		my $minimum_perl = prereqs_for($meta, qw/runtime requires perl 5.006/);
+		my $minimum_perl = prereqs_for($meta, qw/runtime requires perl 5.008/);
 		"# This Build.PL for $dist_name was generated by mbtiny $VERSION.\nuse $minimum_perl;\nuse Module::Build::Tiny $minimum_mbt;\nBuild_PL();\n";
 	};
 	$files{'META.json'} //= $meta->as_string;
